#!/usr/bin/env python
"""Unit tests for standard_names.io module."""
from six.moves import StringIO

from standard_names.utilities import from_model_file

_SINGLE_MODEL_FILE_STREAM = StringIO(
    """
%YAML 1.2
---
model name: topoflow
exchange items:
  - air__density
  - air__emissivity
..."""
)

_MULTIPLE_MODEL_FILE_STREAM = StringIO(
    """
%YAML 1.2
---
model name: topoflow
exchange items:
  - air__density
  - air__emissivity
---
model name: sedflux
exchange items:
  - air__density
  - water__temperature
..."""
)

_SINGLE_MODEL_FILE_STREAM_WITH_INTENT = StringIO(
    """
%YAML 1.2
---
model name: topoflow
exchange items:
  input:
    - air__density
  output:
    - air__density
    - air__emissivity
..."""
)


def test_from_model_file():
    """Read from a YAML model file that contains one model."""
    names = from_model_file(_SINGLE_MODEL_FILE_STREAM)

    assert "air__density" in names
    assert "air__emissivity" in names
    assert len(names) == 2


def test_from_multiple_model_file():
    """Read from a YAML model file that contains multiple models."""
    names = from_model_file(_MULTIPLE_MODEL_FILE_STREAM)

    assert "air__density" in names
    assert "air__emissivity" in names
    assert "water__temperature" in names
    assert len(names) == 3


def test_from_model_file_with_intent():
    """
    Read from a YAML model file that contains one model but that indicates
    intent of variables.
    """
    names = from_model_file(_SINGLE_MODEL_FILE_STREAM_WITH_INTENT)

    assert "air__density" in names
    assert "air__emissivity" in names
    assert len(names) == 2



#! /usr/bin/env python


class Error(Exception):

    """Base class for exceptions from this module."""

    pass


class BadNameError(Error):

    """Error to indicate a poorly-formed standard name."""

    def __init__(self, name):
        super(BadNameError, self).__init__()
        self._name = name

    def __str__(self):
        return self._name

    @property
    def name(self):
        return self._name


class BadRegistryError(Error):

    """Error to indicate a bad NamesRegistry."""

    def __init__(self, names):
        super(BadRegistryError, self).__init__()
        self._names = tuple(sorted(names))

    def __str__(self):
        return "Registry contains invalid names"

    @property
    def names(self):
        return self._names



"""The CSDMS Standard Names"""

from ._version import get_versions
from .error import BadNameError, BadRegistryError
from .registry import NamesRegistry
from .standardname import StandardName, is_valid_name

__all__ = [
    "StandardName",
    "is_valid_name",
    "NamesRegistry",
    "BadNameError",
    "BadRegistryError",
]

__version__ = get_versions()["version"]
del get_versions



#!/usr/bin/env python
"""Unit tests for standard_names.StandardName"""
import pytest

from standard_names import BadNameError, StandardName


def test_create():
    """Test class creation."""
    name = StandardName("air__temperature")

    assert name.name == "air__temperature"


def test_bad_name():
    """Try to create an invalid name."""
    with pytest.raises(BadNameError):
        StandardName("air_temperature")

    with pytest.raises(BadNameError):
        StandardName("air___temperature")

    with pytest.raises(BadNameError):
        StandardName("Air__Temperature")

    with pytest.raises(BadNameError):
        StandardName("_air__temperature")

    with pytest.raises(BadNameError):
        StandardName("air__temperature_")

    with pytest.raises(BadNameError):
        StandardName("air__temperature__0")

    with pytest.raises(BadNameError):
        StandardName("0_air__temperature")


def test_get_object():
    """Retrieve an object from a standard name."""
    name = StandardName("air__temperature")
    assert name.object == "air"


def test_get_quantity():
    """Retrieve a quantity from a standard name."""
    name = StandardName("air__temperature")
    assert name.quantity == "temperature"


def test_get_empty_operator():
    """Retrieve an operator from a standard name."""
    name = StandardName("air__temperature")

    assert name.operators == ()


def test_get_one_operator():
    """Retrieve an operator from a standard name."""
    name = StandardName("air__log_of_temperature")

    assert name.operators == ("log",)


def test_get_multiple_operators():
    """Retrieve an operator from a standard name."""
    name = StandardName("air__mean_of_log_of_temperature")
    assert name.operators == ("mean", "log")


def test_compare_names():
    """Compare a names."""
    name = StandardName("air__temperature")

    assert name == StandardName("air__temperature")
    assert name != StandardName("water__temperature")


def test_compare_name_to_str():
    """Compare a name to a string."""
    name = StandardName("air__temperature")

    assert name == "air__temperature"
    assert name != "water__temperature"

    assert not (name != "air__temperature")
    assert not (name == "water__temperature")


def test_lt_name_to_str():
    """Names are ordered alphabetically."""
    air = StandardName("air__temperature")
    water = StandardName("water__temperature")

    assert air < "water__temperature"
    assert air < water
    assert not ("water__temperature" < air)
    assert not (water < air)


def test_gt_name_to_str():
    """Names are ordered alphabetically."""
    air = StandardName("air__temperature")
    water = StandardName("water__temperature")

    assert not (air > "water__temperature")
    assert not (air > water)
    assert "water__temperature" > air
    assert water > air


def test_hash():
    """Hash based on string."""
    a_bunch_of_names = set()

    a_bunch_of_names.add(StandardName("air__temperature"))
    a_bunch_of_names.add(StandardName("water__temperature"))

    assert len(a_bunch_of_names) == 2

    a_bunch_of_names.add(StandardName("air__temperature"))

    assert len(a_bunch_of_names) == 2

    assert "air__temperature" in a_bunch_of_names
    assert "water__temperature" in a_bunch_of_names
    assert StandardName("air__temperature") in a_bunch_of_names

    assert "surface__temperature" not in a_bunch_of_names



from .decorators import format_as_wiki, format_as_yaml, google_doc, plain_text, url
from .io import FORMATTERS, SCRAPERS, from_list_file, from_model_file, scrape

__all__ = [
    "format_as_wiki",
    "format_as_yaml",
    "google_doc",
    "plain_text",
    "url",
    "FORMATTERS",
    "SCRAPERS",
    "from_list_file",
    "from_model_file",
    "scrape",
]



#! /usr/bin/env python
import os
import warnings
from glob import glob

from six import string_types

from .error import BadNameError, BadRegistryError
from .standardname import StandardName


def load_names_from_txt(file_like, onerror="raise"):
    """Load names from a text file.

    Parameters
    ----------
    file_like : file-like
        A file-like object that represents the contents of a text file
        (only a ``readline`` method need be available).
    onerror : {'raise', 'warn', 'pass'}
        What to do if a bad name is encountered in the file.

    Returns
    -------
    set of str
        The Standard Names read from the file.

    Examples
    --------
    >>> from six.moves import StringIO
    >>> import standard_names as csn
    >>> names = StringIO(\"\"\"
    ... air__temperature
    ... Water__Temperature
    ... \"\"\")
    >>> set_of_names = csn.registry.load_names_from_txt(names, onerror='warn')
    >>> [name.name for name in set_of_names]
    ['air__temperature']
    """
    if onerror not in ("pass", "raise", "warn"):
        raise ValueError("value for onerror keyword not understood")

    bad_names = set()
    names = set()
    for name in file_like:
        name = name.strip()
        if name:
            try:
                csn = StandardName(name)
            except BadNameError:
                bad_names.add(name)
            else:
                names.add(csn)

    if bad_names:
        if onerror == "warn":
            for name in bad_names:
                warnings.warn("{name}: not a valid name".format(name=name))
        elif onerror == "raise":
            raise BadRegistryError(bad_names)

    return names


def _strict_version_or_raise(version_str):
    from distutils.version import StrictVersion

    if StrictVersion.version_re.match(version_str):
        return StrictVersion(version_str)
    else:
        raise ValueError("{version}: Not a version string".format(version=version_str))


def _get_latest_names_file(path=None, prefix="names-", suffix=".txt"):
    """Get the most recent version of a names file.

    Parameters
    ----------
    path : str, optional
        If given, the path to a folder holding names files. Otherwise,
        the default location within the *standard_names* package.
    prefix : str, optional
        The prefix for names-file glob.
    suffix : str, optional
        The suffix for names-file glob.

    Returns
    -------
    tuple of str
        Tuple of the name of the latest file and its version.

    Examples
    --------
    >>> import os
    >>> from standard_names.registry import _get_latest_names_file

    >>> fname, version = _get_latest_names_file()
    >>> os.path.basename(fname)
    'names-0.8.5.txt'
    >>> version
    '0.8.5'

    >>> _get_latest_names_file(prefix='garbage')
    (None, None)

    >>> _get_latest_names_file(prefix='names-0.8.3')
    (None, None)
    """
    data_dir = path or os.path.join(os.path.dirname(__file__), "data")

    name_glob = "{prefix}*{suffix}".format(prefix=prefix, suffix=suffix)
    data_file_pattern = os.path.join(data_dir, name_glob)
    files = [os.path.basename(file_) for file_ in glob(data_file_pattern)]

    newest = None
    for file in files:
        version_str = file[len(prefix) : -len(suffix)]
        try:
            version = _strict_version_or_raise(version_str)
        except ValueError:
            pass
        else:
            if newest is None or version > newest:
                newest = version

    if newest:
        version = str(newest)
        names_file = os.path.join(
            data_dir,
            "{prefix}{version}{suffix}".format(
                prefix=prefix, suffix=suffix, version=version
            ),
        )
        return names_file, version
    else:
        return None, None


class NamesRegistry(object):

    """A registry of CSDMS Standard Names.

    Parameters
    ----------
    paths : str or iterable of str, optional
        Name(s) of the data file(s) from which to read. If not given,
        use a default database. If ``None``, create an empty registry.
    version : str, optional
        The version of the names registry.

    Attributes
    ----------
    version
    names
    objects
    quantities
    operators

    Examples
    --------
    >>> from standard_names import NamesRegistry

    Get the default set of names.

    >>> registry = NamesRegistry()
    >>> len(registry) > 0
    True

    Create an empty registry and add a name to it.

    >>> registry = NamesRegistry(None)
    >>> len(registry)
    0
    >>> registry.add('air__temperature')
    >>> len(registry)
    1

    Use the ``names``, ``objects``, ``quantities``, and ``operators`` to
    get lists of each in the registry.

    >>> registry.names
    ('air__temperature',)
    >>> registry.objects
    ('air',)
    >>> registry.quantities
    ('temperature',)
    >>> registry.operators
    ()

    You can search the registry for names using the ``names_with``,
    ``match``, and ``search`` methods.

    Use ``names_with`` to look for names that contain a given string or
    strings.

    >>> registry.add('water__temperature')
    >>> sorted(registry.names_with('temperature'))
    ['air__temperature', 'water__temperature']
    >>> registry.names_with(['temperature', 'air'])
    ['air__temperature']

    Use ``match`` to match names using a glob-style pattern.

    >>> registry.match('air*')
    ['air__temperature']

    Use ``search`` to do a fuzzy search of the list.

    >>> registry.search('air__temp')
    ['air__temperature']
    """

    def __init__(self, *args, **kwds):
        if len(args) == 0:
            paths, version = _get_latest_names_file()
        elif len(args) == 1:
            paths, version = args[0], None
        else:
            raise ValueError("0 or 1 arguments expected")

        if paths is None:
            paths = []

        if isinstance(paths, string_types) or hasattr(paths, "readline"):
            paths = [paths]

        self._names = set()
        self._objects = set()
        self._quantities = set()
        self._operators = set()

        self._version = version or "0.0.0"

        for path in paths:
            if isinstance(path, string_types):
                with open(path, "r") as fp:
                    self._load(fp)
            else:
                self._load(path)

    def _load(self, file_like, onerror="raise"):
        for name in load_names_from_txt(file_like, onerror=onerror):
            self.add(name)

    @property
    def version(self):
        """The version of the names database.

        Returns
        -------
        str
            The registry version.
        """
        return self._version

    @property
    def names(self):
        """All names in the registry.

        Returns
        -------
        tuple of str
            All of the names in the registry.
        """
        return tuple(self._names)

    @property
    def objects(self):
        """All objects in the registry.

        Returns
        -------
        tuple of str
            All of the objects in the registry.
        """
        return tuple(self._objects)

    @property
    def quantities(self):
        """All quantities in the registry.

        Returns
        -------
        tuple of str
            All of the quantities in the registry.
        """
        return tuple(self._quantities)

    @property
    def operators(self):
        """All operators in the registry.

        Returns
        -------
        tuple of str
            All of the operators in the registry.
        """
        return tuple(self._operators)

    @classmethod
    def from_path(cls, path):
        """Create a new registry from a text file.

        Parameters
        ----------
        path : str
            Path to a text file of Standard Names.

        Returns
        -------
        NamesRegistry
            A newly-created registry filled with names from the file.
        """
        return cls(path)

    def add(self, name):
        """Add a name to the registry.

        Parameters
        ----------
        name : str
            A Standard Name.
        """
        if not isinstance(name, StandardName):
            name = StandardName(name)

        self._names.add(name.name)
        self._objects.add(name.object)
        self._quantities.add(name.quantity)
        for op in name.operators:
            self._operators.add(op)

    def __contains__(self, name):
        if isinstance(name, StandardName):
            name = name.name
        return name in self._names

    def __len__(self):
        return len(self._names)

    def __iter__(self):
        for name in self._names:
            yield name

    def search(self, name):
        """Search the registry for a name.

        Parameters
        ----------
        name : str
            Name to search for.

        Returns
        -------
        tuple of str
            Names that closely match the given name.
        """
        from difflib import get_close_matches

        return get_close_matches(name, self._names)

    def match(self, pattern):
        """Search the registry for names that match a pattern.

        Parameters
        ----------
        pattern : str
            Glob-style pattern with which to search the registry.

        Returns
        -------
        list of str
            List of names matching the pattern.
        """
        import fnmatch
        import re

        p = re.compile(fnmatch.translate(pattern))
        names = []
        for name in self._names:
            if p.match(name):
                names.append(name)
        return names

    def names_with(self, parts):
        """Search the registry for names containing words.

        Parameters
        ----------
        parts : str or iterable of str
            Word(s) to search for.

        Returns
        -------
        tuple of str
            Names from the registry that contains the given words.
        """
        if isinstance(parts, string_types):
            parts = (parts,)

        remaining_names = self._names
        for part in parts:
            names = []
            for name in remaining_names:
                if part in name:
                    names.append(name)
            remaining_names = names
        return names


REGISTRY = NamesRegistry()

NAMES = REGISTRY.names
OBJECTS = REGISTRY.objects
QUANTITIES = REGISTRY.quantities
OPERATORS = REGISTRY.operators
VERSION = REGISTRY.version



#! /usr/bin/env python
"""Some decorators for the CmtStandardNames package."""

import os

from six import string_types


def format_as_wiki(func):
    """
    Decoratate a function that reads lines from a file. Put some wiki
    formatting around each line of the file and add a header, and
    footer.

    Examples
    --------
    >>> from __future__ import print_function
    >>> import os
    >>> from standard_names.utilities.decorators import format_as_wiki

    >>> def func(lines):
    ...     return lines

    >>> wikize = format_as_wiki(func)
    >>> lines = os.linesep.join(['line 1', 'line 2'])
    >>> print(wikize(lines, heading='Lines', newline='\\n'))
    = Lines =
    <tt>
    line 1<br/>
    line 2<br/>
    </tt>
    """

    def _wrapped(lines, **kwds):
        """Decorate a list of strings.

        :lines: List of strings
        :returns: Decorated strings concatoranted with line separators
        """
        newline = kwds.pop("newline", os.linesep)
        heading = kwds.pop("heading", None)
        heading_level = kwds.pop("level", 1)

        text = func(lines, **kwds)
        lines = text.split(os.linesep)

        wiki_lines = []
        for line in lines:
            wiki_lines.append(line + "<br/>")
        wiki_lines.insert(0, "<tt>")
        wiki_lines.append("</tt>")

        if heading:
            pre = "=" * heading_level
            wiki_lines.insert(0, "%s %s %s" % (pre, heading.title(), pre))

        return newline.join(wiki_lines)

    return _wrapped


def format_as_yaml(func):
    """
    Decoratate a function that reads lines from a file. Put some YAML
    formatting around each line of the file and add a header, and
    footer.

    Examples
    --------
    >>> from __future__ import print_function
    >>> import os
    >>> from standard_names.utilities.decorators import format_as_yaml

    >>> def func(lines):
    ...     return lines

    >>> yamlize = format_as_yaml(func)
    >>> lines = os.linesep.join(['line 1', 'line 2'])
    >>> print(yamlize(lines, heading='Lines', newline='\\n'))
    Lines:
      - line 1
      - line 2
    >>> print(yamlize(lines, newline='\\n'))
    - line 1
    - line 2
    """

    def _wrapped(lines, **kwds):
        """Decorate a list of strings.

        Parameters
        ---------
        lines : iterable or str
            List of strings

        Returns
        -------
        str
            Decorated strings concatoranted with line separators
        """
        heading = kwds.pop("heading", None)
        newline = kwds.pop("newline", os.linesep)

        text = func(lines, **kwds)
        lines = text.split(os.linesep)

        if heading:
            yaml_lines = ["%s:" % heading]
            indent = 2
        else:
            yaml_lines = []
            indent = 0

        items = [line for line in lines if line]
        if items:
            for line in items:
                yaml_lines.append("%s- %s" % (" " * indent, line))
        else:
            yaml_lines.append("%s[]" % (" " * indent))

        return newline.join(yaml_lines)

    return _wrapped


def format_as_plain_text(func):
    """

    Examples
    --------
    >>> from __future__ import print_function
    >>> from standard_names.utilities.decorators import format_as_plain_text

    >>> def func(lines):
    ...     return lines

    >>> textize = format_as_plain_text(func)
    >>> lines = os.linesep.join(['line 1', 'line 2'])
    >>> print(textize(lines, heading='Lines', newline='\\n'))
    # Lines
    line 1
    line 2
    """

    def _wrapped(lines, **kwds):
        heading = kwds.pop("heading", None)
        newline = kwds.pop("newline", os.linesep)

        text = func(lines, **kwds)
        lines = text.split(os.linesep)

        if heading:
            stripped_lines = ["# %s" % heading]
        else:
            stripped_lines = []

        for line in lines:
            stripped_lines.append(line.strip())

        return newline.join(stripped_lines)

    return _wrapped


def plain_text(func):
    """
    Decoratate a function that reads from a file-like object. The decorated
    function will instead read from a file with a given name.
    """

    def _wrapped(name, **kwds):
        """Open a file by name.

        Parameters
        ----------
        name : str
            Name of the file as a string.
        """
        if isinstance(name, string_types):
            with open(name, "r") as file_like:
                rtn = func(file_like, **kwds)
        else:
            rtn = func(name, **kwds)
        return rtn

    return _wrapped


def url(func):
    """
    Decoratate a function that reads from a file-like object. The decorated
    function will instead read from a file with a URL.
    """

    def _wrapped(name, **kwds):
        """Open a URL by name.

        Parameters
        ----------
        name : str
            Name of the URL as a string.
        """
        import urllib

        file_like = urllib.urlopen(name)
        rtn = func(file_like, **kwds)
        return rtn

    return _wrapped


def google_doc(func):
    """
    Decoratate a function that reads from a file-like object. The decorated
    function will instead read from a remote Google Doc file.
    """

    def _wrapped(name, **kwds):
        """Open a Google Doc file by name.

        Parameters
        ----------
        name : str
            Name of the Google Doc file as a string.
        """
        import subprocess
        import tempfile

        (_, tfile) = tempfile.mkstemp(text=True)

        try:
            subprocess.check_call(["google", "docs", "get", name, tfile])
        except subprocess.CalledProcessError:
            raise
        else:
            with open(tfile, "r") as file_like:
                rtn = func(file_like, **kwds)
        finally:
            os.remove(tfile)

        return rtn

    return _wrapped



#!/usr/bin/env python


from setuptools import setup

import versioneer


setup(
    name="standard_names",
    version=versioneer.get_version(),
    description="CSDMS standard names",
    author="Eric Hutton",
    author_email="eric.hutton@colorado.edu",
    url="https://csdms.colorado.edu",
    classifiers=[
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Cython",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Scientific/Engineering :: Physics",
    ],
    install_requires=[
        "pyyaml",
        "six",
        "packaging",
        "jinja2",
        "py-scripting",
        "binaryornot",
    ],
    packages=[
        "standard_names",
        "standard_names.cmd",
        "standard_names.utilities",
        "standard_names.tests",
    ],
    cmdclass=versioneer.get_cmdclass(),
    entry_points={
        "console_scripts": [
            "snbuild = standard_names.cmd.snbuild:run",
            "sndump = standard_names.cmd.sndump:run",
            "snscrape = standard_names.cmd.snscrape:run",
            "snsql = standard_names.cmd.snsql:run",
            "snvalidate = standard_names.cmd.snvalidate:run",
        ]
    },
    package_data={"": ["data/*txt"]},
    test_suite="standard_names.tests",
)



#!/usr/bin/env python
"""Unit tests for standard_names.NamesRegistry."""
import pytest
from six import string_types
from six.moves import StringIO

from standard_names import BadNameError, BadRegistryError, NamesRegistry, StandardName
from standard_names.registry import load_names_from_txt


def test_load_names_bad_onerror():
    with pytest.raises(ValueError):
        load_names_from_txt("dummy_arg", onerror="bad_value")


def test_load_names_bad_name_pass():
    file_like = StringIO("air__temperature\nwater_temperature")
    names = load_names_from_txt(file_like, onerror="pass")
    assert names == {"air__temperature"}


def test_load_names_bad_name_raise():
    file_like = StringIO("air__temperature\nwater_temperature")
    with pytest.raises(BadRegistryError, match="Registry contains invalid names"):
        load_names_from_txt(file_like, onerror="raise")

    file_like = StringIO("air__temperature\nwater_temperature")
    try:
        load_names_from_txt(file_like, onerror="raise")
    except BadRegistryError as error:
        assert error.names == ("water_temperature",)


def test_create_full():
    """Test creating default registry."""
    nreg = NamesRegistry()
    assert len(nreg) > 0


def test_create_empty():
    """Test creating default registry."""
    nreg = NamesRegistry(None)
    assert len(nreg) == 0


def test_create_with_too_many_args():
    """Test creating a registry with too many arguments."""
    with pytest.raises(ValueError):
        NamesRegistry("file1", "file2")


def test_create_with_file_like():
    """Test creating a registry from a file_like object."""
    file_like = StringIO("air__temperature")
    names = NamesRegistry(file_like)
    assert names.names == ("air__temperature",)

    file_like = StringIO("air__temperature")
    another_file_like = StringIO("water__temperature")
    names = NamesRegistry([file_like, another_file_like])
    assert isinstance(names.names, tuple)
    assert sorted(names.names) == ["air__temperature", "water__temperature"]


def test_create_with_from_path():
    """Test creating registry with from_path."""
    file_like = StringIO("air__temperature")
    names = NamesRegistry.from_path(file_like)
    assert names.names == ("air__temperature",)


def test_bad_name():
    """Try to add an invalid name."""
    nreg = NamesRegistry(None)
    with pytest.raises(BadNameError):
        nreg.add("air_temperature")


def test_add_string():
    """Add a string to the collection."""
    nreg = NamesRegistry(None)
    nreg.add("air__temperature")

    assert StandardName("air__temperature") in nreg
    assert "air__temperature" in nreg
    assert len(nreg) == 1

    nreg.add("air__temperature")
    assert len(nreg) == 1


def test_collection_contains_names():
    """Make sure a NamesRegistry only contains StandardName objects."""
    nreg = NamesRegistry(None)
    nreg.add("air__temperature")
    nreg.add("water__temperature")

    for name in nreg:
        assert isinstance(name, string_types)


def test_add_name():
    """Add a name to the collection."""
    nreg = NamesRegistry(None)
    nreg.add(StandardName("air__temperature"))

    assert StandardName("air__temperature") in nreg
    assert "air__temperature" in nreg


def test_unique_names():
    """List of unique names."""
    nreg = NamesRegistry(None)
    nreg.add("air__temperature")
    nreg.add("water__temperature")

    assert len(nreg) == 2

    assert "air__temperature" in nreg
    assert "water__temperature" in nreg


def test_unique_objects():
    """List of unique objects."""
    nreg = NamesRegistry(None)
    nreg.add("air__temperature")
    nreg.add("water__temperature")

    objs = nreg.objects

    assert len(objs) == 2
    assert "air" in objs
    assert "water" in objs


def test_unique_quantities():
    """List of unique quantities."""
    nreg = NamesRegistry(None)
    nreg.add("air__temperature")
    nreg.add("water__temperature")

    quantities = nreg.quantities

    assert quantities == ("temperature",)


def test_unique_operators():
    """List of unique operators."""
    nreg = NamesRegistry(None)
    nreg.add("air__temperature")
    nreg.add("water__temperature")
    nreg.add("air__log_of_temperature")
    nreg.add("air__mean_of_log_of_temperature")

    operators = nreg.operators

    assert len(operators) == 2
    assert "log" in operators
    assert "mean" in operators
